#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or(at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2003 Free Software Foundation
#
# FILE:
# _pgsql/DBdriver.py
#
# DESCRIPTION:
# A core Postgresql dbdriver that the other (specific)
# postgresql drivers can extend
#
# NOTES:
#

import string
from string import lower, join, split
import sys
from gnue.common.apps import GDebug, GConfig
from gnue.common.datasources import GDataObjects
from gnue.common.datasources.drivers.DBSIG2.Driver \
   import DBSIG_RecordSet, DBSIG_ResultSet, DBSIG_DataObject

from gnue.common.datasources.drivers.DBSIG2.Driver import DBSIG_DataObject_Object as PGSQL_DataObject_Object
from gnue.common.datasources.drivers.DBSIG2.Driver import DBSIG_DataObject_SQL as PGSQL_DataObject_SQL


class PGSQL_RecordSet(DBSIG_RecordSet):
  def _buildUpdateStatement(self):
    updates = []
    for field in self._modifiedFlags.keys():
      updates.append ("%s=%s" % (field,
        self._parent._dataObject._toSqlString(self._fields[field])))

    if self._parent._dataObject._primaryIdField:
      where = [self._parent._dataObject._primaryIdFormat % \
          self._initialData[self._parent._dataObject._primaryIdField]  ]
    else:
      where = []
      for field in self._initialData.keys():
        if self._initialData[field] == None:
          where.append ("%s IS NULL" % field)
        else:
          where.append ("%s=%s" % (field, self._parent._dataObject._toSqlString(self._initialData[field])))
    return "UPDATE %s SET %s WHERE %s" % \
       (self._parent._dataObject.table, string.join(updates,','), \
        string.join(where,' AND ') )


class PGSQL_ResultSet(DBSIG_ResultSet):
  def __init__(self, dataObject, cursor=None, defaultValues={}, masterRecordSet=None):
    DBSIG_ResultSet.__init__(self, dataObject, \
                             cursor, defaultValues, masterRecordSet)
    self._recordSetClass = PGSQL_RecordSet

class PGSQL_DataObject(DBSIG_DataObject):

  encodingList={'ascii'     :  'SQL_ASCII',     # ASCII
                ''          :  'EUC_JP',        # Japanese EUC
                ''          :  'EUC_CN',	# Chinese EUC
                ''          :  'EUC_KR',	# Korean EUC
                ''          :  'JOHAB',	        # Korean EUC (Hangle base)
                ''          :  'EUC_TW',	# Taiwan EUC
                'utf-8'     :  'UNICODE',       # Unicode (UTF-8)
                ''          :  'MULE_INTERNAL', # Mule internal code
                'iso8859-1' :  'LATIN1',	# ISO 8859-1 ECMA-94 Latin Alphabet No.1
                'iso8859-2' :  'LATIN2',	# ISO 8859-2 ECMA-94 Latin Alphabet No.2
                'iso8859-3' :  'LATIN3',	# ISO 8859-3 ECMA-94 Latin Alphabet No.3
                'iso8859-4' :  'LATIN4',	# ISO 8859-4 ECMA-94 Latin Alphabet No.4
                'iso8859-9' :  'LATIN5',	# ISO 8859-9 ECMA-128 Latin Alphabet No.5
                'iso8859-10':  'LATIN6',	# ISO 8859-10 ECMA-144 Latin Alphabet No.6
                'iso8859-13':  'LATIN7',	# ISO 8859-13 Latin Alphabet No.7
                'iso8859-14':  'LATIN8',	# ISO 8859-14 Latin Alphabet No.8
                'iso8859-15':  'LATIN9',	# ISO 8859-15 Latin Alphabet No.9
                'iso8859-16':  'LATIN10',	# ISO 8859-16 ASRO SR 14111 Latin Alphabet No.10
                'iso8859-5' :  'ISO-8859-5',    # ECMA-113 Latin/Cyrillic
                'iso8859-6' :  'ISO-8859-6',    # ECMA-114 Latin/Arabic
                'iso8859-7' :  'ISO-8859-7',    # ECMA-118 Latin/Greek
                'iso8859-8' :  'ISO-8859-8',    # ECMA-121 Latin/Hebrew
                ''          :  'KOI8',          # KOI8-R(U)
                ''          :  'WIN',           # Windows CP1251
                ''          :  'ALT',           # Windows CP866
                ''          :  'WIN1256',       # Arabic Windows CP1256
                ''          :  'TCVN',          # Vietnamese TCVN-5712 (Windows CP1258)
                ''          :  'WIN874'}        # Thai Windows CP874

  def __init__(self, pgdriver=None, pgresultset=None):
    DBSIG_DataObject.__init__(self)
    self._connectString = 'host=%s dbname=%s user=%s password=%s port=%s'
    self._escapeSingleQuote = '\\'
    # date/time format
    self._dateTimeFormat = "'%Y-%m-%d %H:%M:%S'"

    if pgdriver:
      self._pgdriver = pgdriver
      self._DatabaseError = self._pgdriver.DatabaseError
    if pgresultset:
      self._resultSetClass = pgresultset

  def connect(self, connectData={}):
    GDebug.printMesg(1,"Postgresql database driver initializing")
    try:
      try:
        port = connectData['port']
      except:
        port  = '5432'

      self._dataConnection = self._pgdriver.connect(self._connectString %
                                                    (connectData['host'],
                                                     connectData['dbname'],
                                                     connectData['_username'],
                                                     connectData['_password'],
                                                     port))
      self.triggerExtensions = PGSQL_TriggerExtensions(self._dataConnection)
    except self._DatabaseError, value:
      GDebug.printMesg(1,"Connect String: %s" % (self._connectString % \
             (connectData['host'],
              connectData['dbname'],
              connectData['_username'],
              connectData['_password'],
              port)))
      GDebug.printMesg(1,"Exception %s " % value)
      raise GDataObjects.LoginError, value

    try:
      try:
        self._databaseEncoding = connectData['encoding']
      except KeyError:
        pass

      try:
        encoding = self.encodingList[self._databaseEncoding]
      except KeyError:
        GDebug.printMesg(1,_("Encoding '%s' is not supported by postgresql dbdriver.") % \
                         self._databaseEncoding)
        encoding = ''
        
      if encoding.upper() not in ("",'DEFAULT'):
        GDebug.printMesg(1,'Setting postgresql client_encoding to %s (%s)' % (encoding,
                                                                              self._databaseEncoding))
        cursor = self._dataConnection.cursor()
        cursor.execute("SET CLIENT_ENCODING TO '%s'" % encoding)
        cursor.close()

    except self._DatabaseError:
      try:
        cursor.close()
      except:
        pass

    if connectData.has_key('datetimeformat'):
      self._dateTimeFormat = "'%s'" % connectData['datetimeformat']


    self._postConnect()

  def _postConnect(self):
    self.triggerExtensions = PGSQL_TriggerExtensions(self._dataConnection)

  #
  # Schema (metadata) functions
  #

  # Return a list of the types of Schema objects this driver provides
  def getSchemaTypes(self):
    return [('view',_('Views'),1),
            ('table',_('Tables'),1)]

  # Return a list of Schema objects
  def getSchemaList(self, type=None):
    includeTables = (type in ('table','sources', None))
    includeViews = (type in ('view','sources', None))

    inClause = []
    if includeTables:
      inClause.append ("'r'")
    if includeViews:
      inClause.append ("'v'")

    # TODO: This excludes any system tables and views. Should it?
    statement = "select relname, relkind, oid from pg_class " + \
            "where relkind in (%s) " % (join(inClause,',')) + \
            "and relname not like 'pg_%' " + \
            "order by relname"

    cursor = self._dataConnection.cursor()
    cursor.execute(statement)

    list = []
    for rs in cursor.fetchall():
      list.append(GDataObjects.Schema(attrs={'id':rs[2], 'name':rs[0],
                         'type':rs[1] == 'v' and 'view' or 'table',
                         'primarykey': self.__getPrimaryKey(cursor, rs[2])},
                         getChildSchema=self.__getFieldSchema))

    cursor.close()
    return list


  # Find a schema object with specified name
  def getSchemaByName(self, name, type=None):
    statement = "select relname, relkind, oid from pg_class " + \
            "where relname = '%s'" % (name)

    cursor = self._dataConnection.cursor()
    cursor.execute(statement)

    rs = cursor.fetchone()
    if rs:
      schema = GDataObjects.Schema(attrs={'id':rs[2], 'name':rs[0],
                           'type':rs[1] == 'v' and 'view' or 'table',
                           'primarykey': self.__getPrimaryKey(cursor, rs[2]) },
                           getChildSchema=self.__getFieldSchema)
    else:
      schema = None

    cursor.close()
    return schema

  def __getPrimaryKey(self, cursor, oid):
    cursor = self._dataConnection.cursor()
    cursor.execute("select indkey from pg_index where indrelid=%d" % oid)
    rs = cursor.fetchone()
    statement = "select attname from pg_attribute " \
                "where attrelid = %d and attnum = %%d" % oid
    if rs:
      pks = []
      for indpos in string.split(rs[0]):
        cursor.execute(statement % int(indpos))
        pks.append(cursor.fetchone()[0])
      cursor.close()
      return tuple(pks)
    else:
      cursor.close()
      return None

  # Get fields for a table
  def __getFieldSchema(self, parent):

    statement = "select attname, pg_type.oid, typname, " + \
            " attnotnull, atthasdef, atttypmod, attnum, attlen " + \
            "from pg_attribute, pg_type " + \
            "where attrelid = %d and " % (parent.id) + \
            "pg_type.oid = atttypid and attnum >= 0" + \
            "order by attnum"

    cursor = self._dataConnection.cursor()
    cursor.execute(statement)

    list = []
    for rs in cursor.fetchall():

      attrs={'id': rs[1], 'name': rs[0],
             'type':'field', 'nativetype': rs[2],
             'required': rs[3] and not rs[4]}

      if rs[2] in ('numeric','float4','float8','money','bool','int8','int2','int4'):
        attrs['datatype']='number'
      elif rs[2] in ('date','time','timestamp','abstime','reltime'):
        attrs['datatype']='date'
      else:
        attrs['datatype']='text'

      if rs[7] > 0:
        attrs['length'] = rs[7]
      elif rs[5] != -1: #text field
        attrs['length'] = rs[5] - 4


      # Find any default values
      if rs[4]:
        cursor.execute("select adsrc " + \
                       "from pg_attrdef " + \
                       "where adrelid = %d and adnum = %d" % (parent.id, rs[6]))
        defrs = cursor.fetchone()
        if defrs:
          dflt = defrs[0]
          if dflt[:8] == 'nextval(':
            attrs['defaulttype'] = 'sequence'
            attrs['defaultval'] = split(dflt,"'")[1]
          elif dflt == 'now()':
            attrs['defaulttype'] = 'system'
            attrs['defaultval'] = 'timestamp'
          else:
            attrs['defaulttype'] = 'constant'
            attrs['defaultval'] = dflt

      list.append(GDataObjects.Schema(attrs=attrs))

    cursor.close()
    return list
   
#
#  Extensions to Trigger Namespaces
#
class PGSQL_TriggerExtensions:

  def __init__(self, connection):
    self.__connection = connection

  # Return the current date, according to database
  def getTimeStamp(self):
    return self.__singleQuery("select current_timestamp")

  # Return a sequence number from sequence 'name'
  def getSequence(self, name):
    return self.__singleQuery("select nextval('%s')" % name)

  # Run the SQL statement 'statement'
  def sql(self, statement):
    cursor = self.__connection.cursor()
    try:
      cursor.execute(statement)
      cursor.close()
    except:
      cursor.close()
      raise

  # Used internally
  def __singleQuery(self, statement):
    cursor = self.__connection.cursor()
    try:
      cursor.execute(statement)
      rv = cursor.fetchone()
      cursor.close()
    except:
      print "DBdriver.py", "You've got your bar in my foo! And you've got your foo on my bar!  Two great reams that ream well together!"
      GDebug.printMesg(1,"**** Unable to execute extension query")
      GDebug.printMesg(1,"**** %s" % sys.exc_info()[1])
      cursor.close()
      raise

    try:
      return rv[0]
    except:
      return None



