#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2003 Free Software Foundation
#
# FILE:
# EventController.py
#
# DESCRIPTION:
#
# NOTES:
#

from gnue.common.apps import GDebug
import string

from EventAware import EventAware
from Event import Event

class EventController(EventAware):

  def __init__(self):
    # Note: Don't call EventAware.__init__
    #   ... that would be fugly.

    # Store a dictionary of registered events
    self.__incomingEvents = {}


  def registerEventListeners(self, events):
    for event in events.keys():
      try:
        self.__incomingEvents[event].append(events[event])
      except KeyError:
        self.__incomingEvents[event] = [events[event]]

  def startCachingEvents(self):
    self.__cache = []

  def stopCachingEvents(self):
    cache = self.__cache
    del self.__cache
    for event, arg, parms in cache:
      self.dispatchEvent(event, *arg, **parms)

  def dispatchEvent(self, event, *args, **parms):

    # If we are caching our events, cache it:
    try:
      self.__cache.append((event, args, parms))
      return
    except AttributeError:
      pass

    # Hackery so dispatchEvent can be passed
    # either an Event() object, or a text string
    # identifying the type of event.  If the
    # latter, an event is created on the fly.
    try:
      event.__event__
    except:
      event = Event(event, *args, **parms)

    #GDebug.printMesg(0, "Event: %s" % (event.__event__))
    try:
      handlers = self.__incomingEvents[event.__event__]
    except KeyError:
      GDebug.printMesg(3, "Unknown Event: %s " % (event.getEvent()))
      return

    for handler in handlers:
      ## GDebug.printMesg(10, "Event: %s in %s" % (event.__event__, handler))
      handler(event)
      if event.__error__ or event.__dropped__:
        break

    # Fire any "dispatchAfter" events
    for args, parms in event.__after__:
      self.dispatchEvent(*args, **parms)

    return event.__result__








