#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2003 Free Software Foundation
#
# FILE:
# xmlrpc/ServerAdapter.py
#
# DESCRIPTION:
# Set of classes that implement the XML-RPC server driver for GNUe Comm.
#
# NOTES:
# Requires py-xmlrpc from http://sf.net/projects/py-xmlrpc
# 
#
# Server Parameters:
#
#    port        The port that the service is located on
#    loglevel    Loglevel of the internal server
#



##############################################################################
#
# We provide a server driver...
#
SERVER = 1      # ServerAdapter


#from gnue.common.rpc.drivers._helpers.AsyncSocketServer import AsyncHTTPServer


from gnue.common.rpc import GComm
from gnue.common.apps import GDebug
from gnue.common.rpc.drivers._helpers import ObjectLibrarian, ObjectEnabler, DirectoryServer
from gnue.common.rpc import GCommBase

import string, sys, thread, traceback

try:
  import xmlrpc

except ImportError:
  tmsg = _("\nUnable to load xmlrpc.  To use the XML-RPC interface, \n") \
         + _("please install xmlrpc from:\n") \
         + "    http://sourceforge.net/projects/py-xmlrpc" \
         + _("the appropriate DEBIAN package is python-xmlrpc")
  raise GComm.AdapterInitializationError, tmsg


# Mapping from GRPC's datatype to XML-RPC datatypes
_datatypeMap = {
  'integer': 'int',
  'string': 'string',
  'boolean': 'boolean',
  'date': 'dateTime.iso8601',
  'number': 'double',
  'base64': 'base64',
  'binary': 'base64'
}


##############################################################################
#
# ServerAdapter
#
class ServerAdapter(ObjectEnabler.ObjectEnabler):

  def __init__(self, rpcdef, bindings, params):

    ## Check parameters
    try:
      self._port = params['port']
    except KeyError:
      tmsg = _('Required parameter "port" not supplied')
      raise GComm.AdapterConfigurationError, tmsg

    # Create Subserver
    self.server=XMLRPC_Server()

    
    ## Build up Service Directory and enable object dispatching
    ObjectEnabler.ObjectEnabler.__init__(self, rpcdef, \
                                         bindings, params)
    
    if hasattr(params,'loglevel'):
      self._loglevel = params['loglevel']
    else:
      self._loglevel = 1
      
    xmlrpc.setLogLevel(self._loglevel)
    
    self.server.setParent(self)
    self.server.bindAndListen(self._port)
    self.server.setOnErr(self.OnErr)

  def serve(self):
    quit=0
    while quit==0:
      try:
	self.server.work()
	# you could set a timeout if desired
      except:
        e = sys.exc_info()
        if e[0] in (KeyboardInterrupt, SystemExit):
          # shut down server without showing error message
          quit=1
          # raise e[0], e[1], e[2]
        else:
          print "*" * 60
          print "Error occured on server:"
          print "*" * 60
          traceback.print_exc()
          print "*" * 60
          print "\n\n"

  def addRpMethod(self,object,parent,binding):    

    DirectoryServer.DirectoryServer.addRpMethod(self,object, \
                                                parent,binding)
    self.server.addMethods({object._path:binding})


  def addRpAttribut(self,object,parent,binding):

    DirectoryServer.DirectoryServer.addRpAttribut(self,object,\
                                                  parent,binding)

    if not object.readonly:
      # Add the set_* directory entry
      self.server.addMethods({'%s.set_%s' % \
                              (parent._path, object.name):\
                              binding})

    if not object.writeonly:
      # Add the get_* directory entry
      self.server.addMethods({'%s.get_%s' % \
                              (parent._path, object.name):\
                              binding})


  def OnErr(self,server,source):
    print _("Internal server error occured:\n"+
            " server %s \n on source %s.") % (server,source)
    return xmlrpc.ONERR_KEEP_WORK
    

  def registerDynamicObject(self,objhandle,type):
    #
    #  add new methods to the xmlrpc server
    #
    tl=len(type)
    for i in self.directory.keys():
      if i[0:tl]==type:
        method="["+objhandle+"]"+i[tl:]
        GDebug.printMesg(1,'Method %s registered to py-xmlrpc ' \
                         % method +\
                         ' internal directory.')
        self.server.addMethods({method:None})
     
    # add a method to close this object
    self.server.addMethods({"["+objhandle+"]._close":None})

  def deregisterDynamicObject(self,object):
    #
    #  remove the new methods from the xmlrpc server
    #
    objhandle=ObjectLibrarian.getObjectReference(object)
    type=object._type
    tl=len(type)
    for i in self.directory.keys():
      if i[0:tl]==type:
        method="["+objhandle+"]"+i[tl:]
        GDebug.printMesg(1,'Method %s is deleted from py-xmlrpc ' \
                         % method +\
                         ' internal directory.')
        self.server.removeMethods({method:None})

    # remove the method to close this object
    self.server.removeMethods({"["+objhandle+"]._close":None}) 
    

class XMLRPC_Server(xmlrpc.server):
  def setParent(self, parent):
    self._parent=parent

  def dispatch(self, servp, srcp, uri, method, params):
    return self._parent.call(method,params)

  def removeMethods(self, dict):
    d = {}
    for (name, func) in dict.items():
      del self.comtab[name]
      # TODO: remove method from internal comtab too
      # del self._o.comtab[name]
      # or
      # self._o.removeMethods(d)
