#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2002-2003 Free Software Foundation
#
# FILE:
# base.py
#
# DESCRIPTION:
# PostgreSQL
#
# NOTES:
#

import string

class BaseProcessor:

  MAX_NAME_LENGTH = 30

  COMMENT_BEGIN = "-- "
#  COMMENT_END = " */\n"
  COMMENT_END = ""
  COMMENT_SINGLELINE=1

  def __init__(self):
    self.prepare()

  def prepare(self):
    pass

  def number(self, attributes={}):
    raise "Unsupported datatype: number"

  def string(self, attributes={}):
    raise "Unsupported datatype: string"

  def date(self, attributes={}):
    raise "Unsupported datatype: date"

  def time(self, attributes={}):
    raise "Unsupported datatype: time"

  def timestamp(self, attributes={}):
    raise "Unsupported datatype: timestamp"

  def text(self, attributes={}):
    raise "Unsupported datatype: text"

  def key(self, attributes={}):
    raise "Unsupported datatype: key"

  def comment(self, destination, comment):

    if self.COMMENT_SINGLELINE:

      for line in comment.split('\n'):
        destination.write("%s%s%s\n" % (self.COMMENT_BEGIN, line, self.COMMENT_END))

    else:
      lead = "\n " * len(self.COMMENT_BEGIN)
      destination.write(self.COMMENT_BEGIN)
      first = 1
      for line in comment.split('\n'):
        if not first:
          destination.write(lead)

        destination.write("%s" % (line))
        first = 0
      destination.write(self.COMMENT_END)


  def createTable(self, name, fields, pretable=[], posttable=[]):
    if pretable:
      rv = string.join(pretable,';\n') + ';\n'
    else:
      rv = ""
    rv += "CREATE TABLE %s\n (" % name
    rv += string.join(fields,',\n  ') + ');\n'
    if posttable:
      rv += "\n" + string.join(posttable,'\n') + '\n'

    return rv


  def createField(self, name, tablename, datatype, object):
    rv = '%s %s' % (name, datatype)
    return ([rv], None, None, None)


  def createPrimaryKey(self, name, tablename, fields):
    rv = 'PRIMARY KEY (%s)' % string.join(fields,', ')
    return (None, [rv], None, None)


  def createIndex(self, name, tablename, fields, unique):
    rv = 'CREATE %sINDEX %s ON %s (%s);' % (unique and "UNIQUE " or "",
                           name, tablename, string.join(fields,', '))
    return (None, None, None, [rv])



