#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2002-2003 Free Software Foundation
#
# FILE:
# interbase.py
#
# DESCRIPTION:
# Interbase
#
# NOTES:
#


from base import BaseProcessor

name = "Interbase"
description = "Interbase (5.x/6.x)/Firebird (1.x)"


class Processor(BaseProcessor):


  COMMENT_BEGIN = "/*"
  COMMENT_END = "*/"
  MAX_NAME_LENGTH = 31

  def prepare(self):
    self.trigBody = []

  def number(self, object):
    if object.precision == 0:
      if object.length <= 4:
        return "smallint"
      elif object.length <= 9:
        return "integer"
      else:
        return "decimal(%s,0)" % object.length
    else:
      return "numeric(%s,%s)" % (object.length + object.precision, object.precision)

  def string(self, object):
    return "varchar(%s)" % object.length

  def date(self, object):
    return "date"

  def time(self, object):
    return "date"

  def timestamp(self, object):
    return "date"

  def text(self, object):
    if hasattr(object,'length') and object.length <= 2000:
      return "varchar(%s)" % object.length
    else:
      return "blob"

  def key(self, object):
    return "integer"


  def createField(self, name, tablename, datatype, object):
    rv = '%s %s' % (name, datatype)

    posttable = []
    pretable = []
    trigBody = self.trigBody

    if object.defaultwith == "serial":

      gen = tablename + '_' + name + '_gen'
      if len(gen) > self.MAX_NAME_LENGTH:
        gen = tablename + "_gen_"
        gen += str(object._parent._children.index(object))
      if len(gen) > self.MAX_NAME_LENGTH:
        gen = tablename + "_" + str(id(object)) + '_gen'
      if len(gen) > self.MAX_NAME_LENGTH:
        gen = 'gen_%s' % id(object)

      trigBody.append(" new.%s=gen_id(%s,1);\n" % (name,gen))

      pretable.append ('CREATE GENERATOR %s' % gen)

    elif object.defaultwith == "timestamp":
      rv += " DEFAULT 'NOW'"

    elif hasattr(object, 'default') and len(object.default):
      rv += ' DEFAULT %s' % object.default

    if not object.nullable:
      rv += ' NOT NULL'

    # Field def, Post-Field Def, Pre-Table command, Post-Table command
    return (rv,None,pretable or None,posttable or None)

  def createTable(self, name, fields, pretable=[], posttable=[]):
    trigBody = self.trigBody

    if len(trigBody):
      posttable.append ("SET TERM ^ ;\n"+\
        "CREATE TRIGGER trg_%s FOR %s ACTIVE BEFORE INSERT" % (name,name)+\
        " POSITION 0 AS BEGIN\n" + ''.join(trigBody)+\
        "END ^\n"+\
        "SET TERM ;^")

    self.prepare()

    return BaseProcessor.createTable(self, name, fields, pretable, posttable)
