#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2002-2003 Free Software Foundation
#
# FILE:
# mysql.py
#
# DESCRIPTION:
# MySQL
#
# NOTES:
#


from base import BaseProcessor

name = "MySQL"
description = "MySQL (3.x/4.x)"


class Processor(BaseProcessor):


  MAX_NAME_LENGTH = 64

  def number(self, object):
    if object.precision == 0:
      if object.length <= 4:
        return "smallint"
      elif object.length <= 9:
        return "int"
      elif object.length <= 18:
        return "bigint"
      else:
        return "decimal(%s,0)" % object.length
    else:
      return "decimal(%s,%s)" % (object.length, object.precision)

  def string(self, object):
    return "varchar(%s)" % object.length

  def date(self, object):
    return "date"

  def time(self, object):
    return "time"

  def timestamp(self, object):
    return "timestamp"

  def text(self, object):
    if hasattr(object,'length') and object.length <= 255:
      return "varchar(%s)" % object.length
    else:
      return "text"

  def key(self, object):
    return "int unsigned"


  def createField(self, name, tablename, datatype, object):
    rv = '%s %s' % (name, datatype)

    posttable = []
    pretable = []

    if object.defaultwith == "serial":

      rv += " auto_increment"

    elif object.defaultwith == "timestamp":
      rv += " DEFAULT `current_timestamp()`"

    elif hasattr(object, 'default') and len(object.default):
      rv += ' DEFAULT %s' % object.default

    if not object.nullable:
      rv += ' NOT NULL'

    # Field def, Post-Field Def, Pre-Table command, Post-Table command
    return (rv,None,pretable or None,posttable or None)

