
name = "Oracle"
description = "Oracle (7.x/8i/9i+)"

from base import BaseProcessor
import string

_PK_PRECISION = 10

class Processor(BaseProcessor):


  COMMENT_BEGIN = "REM ** "
  MAX_NAME_LENGTH = 31

  def prepare(self):
    self.trigWhen = []
    self.trigBody = []


  def number(self, object):
    if object.precision == 0:
      return "number(%s)" % object.length
    else:
      return "number(%s,%s)" % (object.length + object.precision, object.precision)

  def string(self, object):
    return "varchar2(%s)" % object.length

  def date(self, object):
    return "date"

  def time(self, object):
    return "date"

  def timestamp(self, object):
    return "date"

  def text(self, object):
    if hasattr(object,'length') and object.length <= 2000:
      return "varchar2(%s)" % object.length
    else:
      return "long"

  def key(self, object):
    return "number(%s)" % _PK_PRECISION


  def createField(self, name, tablename, datatype, object):
    rv = '%s %s' % (name, datatype)

    posttable = []
    pretable = []
    trigWhen = self.trigWhen
    trigBody = self.trigBody

    if object.defaultwith == "serial":

      sq = tablename + '_' + name + '_seq'
      if len(sq) > self.MAX_NAME_LENGTH:
        sq = tablename + "_seq_"
        sq += str(object._parent._children.index(object))
      if len(sq) > self.MAX_NAME_LENGTH:
        sq = tablename + "_" + str(id(object)) + '_seq'
      if len(sq) > self.MAX_NAME_LENGTH:
        sq = 'seq_%s' % id(object)

      trigWhen.append('(new.%s is null)' % object.name)
      trigBody.append('     IF :new.%s IS NULL THEN\n' % (object.name) + \
                      '        SELECT %s.nextval INTO :new.%s FROM dual;\n' % (sq, object.name) + \
                      '     END IF;')

      pretable.append ('CREATE SEQUENCE %s MAXVALUE %s NOCYCLE' % (sq, '9'*_PK_PRECISION))

    elif object.defaultwith == "timestamp":
      sysdate = object.type == 'date' and "TRUNC(sysdate)" or "sysdate"
      trigWhen.append('(new.%s is null)' % object.name)
      trigBody.append('     IF :new.%s IS NULL then\n' % (object.name) + \
                      '        :new.%s := %s;\n' % (object.name, sysdate) + \
                      '     END IF;')

    elif hasattr(object, 'default') and len(object.default):
      rv += ' DEFAULT %s' % object.default

    if not object.nullable:
      rv += ' NOT NULL'

    # Field def, Post-Field Def, Pre-Table command, Post-Table command
    return (rv,None,pretable or None,posttable or None)

  def createTable(self, name, fields, pretable=[], posttable=[]):
    trigWhen = self.trigWhen
    trigBody = self.trigBody
    if len(trigBody):
      if len(trigWhen):
        when = "  WHEN ( %s )\n" % (string.join(trigWhen,' OR\n         '))
      else:
        when = ""

      posttable.append ('CREATE OR REPLACE TRIGGER t__%s__pre\n' % name + \
                        '  BEFORE INSERT ON %s\n' % name + \
                        '  FOR EACH ROW\n' + \
                        when + \
                        '  BEGIN\n' + \
                        string.join(trigBody,'\n') + '\n' +\
                        '  END;\n/\n')

    self.prepare()

    return BaseProcessor.createTable(self, name, fields, pretable, posttable)

